import { bindable } from 'aurelia-templating';
import { Repository, Entity } from 'aurelia-orm';
import DatatableConfiguration from '../configuration/datatable-configuration';
import DOMUtils from '../utils/DOM-utils';
import DataGenerator from '../utils/data-generator';
import DatatableConfigurator from '../configuration/datatable-configurator';

import * as $ from 'jquery';
import 'datatables.net';
import DatatableColumnsSettings from '../configuration/datatable-columns-settings';

export const DATATABLE_PREFIX = 'datatable';
export const JOIN_CHAR = '-';
export const DATATABLE_ID = DATATABLE_PREFIX + JOIN_CHAR;
export const ERROR_INVALID_ID = 'The given id is not valid:';

/**
 * Datatable component view-model that allow the use
 * of aurelia-orm with jQuery dataTables.
 *
 * @export
 * DatatableJquery
 */
export class DatatableJquery {
  @bindable
  public repository: Repository;

  @bindable
  public columns: DatatableColumnsSettings[];

  // @bindable
  // public buttons: DatatableButtons;

  // @bindable
  // public responsive: DatatableResponsive;

  private _data: Entity[];
  private _id: number;

  private _datatableConfigurator: DatatableConfigurator;
  private _datatableConfiguration: DatatableConfiguration;

  private _table: any;

  private _domUtils: DOMUtils = new DOMUtils();

  public constructor() {
    this._datatableConfiguration = new DatatableConfiguration();
    this._datatableConfigurator = new DatatableConfigurator();

    this._data = [];
    this._id = -1;
  }

  public attached(): void {
    this.configureDatatable();

    this.initializeDatatable();

    this.fetchData();
  }

  /**
   * Configure datatable as user want.
   *
   * @private
   * @memberof DatatableJquery
   */
  private configureDatatable(): void {
    this._datatableConfiguration = this._datatableConfigurator.generateDatatableConfiguration(
      this.repository,
      this.columns
      // this.buttons,
      // this.responsive
    );
  }

  /**
   * Initialize a Datatables instance.
   *
   * @private
   * @memberof DatatableJquery
   */
  private initializeDatatable(): void {
    this._id = this.generateId();

    // let datatablesSettings: DataTables.Settings = new DataTables.Settings();

    this._table = $('#' + this._id).DataTable(this._datatableConfiguration);
    // .DataTable(this._datatableConfiguration);
  }

  /**
   * Generate an id that can be used to instanciate a new datatables.
   *
   * @private
   * @returns {number}
   * @memberof DatatableJquery
   */
  private generateId(): number {
    let usedIds: number[] = this.getUsedIds();

    let desiredId = DataGenerator.generateNumberThatIsNotIn(usedIds);

    return desiredId;
  }

  /**
   * Get all ids used by other datatables.
   *
   * @private
   * @returns {number[]}
   * @memberof DatatableJquery
   */
  private getUsedIds(): number[] {
    let instanciatedDatatables = this.getInstanciatedDatatables();

    return this._domUtils.extractNumbersFromIdsOfNodes(
      DATATABLE_ID,
      instanciatedDatatables
    );
  }

  /**
   * Get all DOM nodes that have an id that begin with DATATABLE_ID.
   *
   * @private
   * @returns {NodeListOf<Element>}
   * @memberof DatatableJquery
   */
  private getInstanciatedDatatables(): NodeListOf<Element> {
    return this._domUtils.getAllNodesWithIdThatBeginsWith(DATATABLE_ID);
  }

  private fetchData(): void {}

  // GETTERS AND SETTERS
  public get datatableConfiguration(): DatatableConfiguration {
    return this._datatableConfiguration;
  }

  public get id(): string {
    return DATATABLE_ID + this._id;
  }

  public get data(): any[] {
    return this._data;
  }
}
