export const EXTRACT_NUMBER_ERROR = 'The given id is not valid: ';

export default class DOMUtils {
  /**
   * Extract numbers from ids of a node list.
   *
   * @param {string} stringToRemove
   * @param {NodeListOf<Element>} nodeList
   * @returns {number[]}
   * @memberof DOMUtils
   */
  public extractNumbersFromIdsOfNodes(
    stringToRemove: string,
    nodeList: NodeListOf<Element>
  ): number[] {
    let extractedNumbers: number[] = [];

    nodeList.forEach(element => {
      extractedNumbers.push(
        this.extractNumberFromId(stringToRemove, element.id)
      );
    });

    return extractedNumbers;
  }

  /**
   * Extract a number from a string.
   *
   * @param {string} stringToRemove
   * @param {string} id
   * @returns {number}
   * @memberof DOMUtils
   */
  public extractNumberFromId(stringToRemove: string, id: string): number {
    /* Don't use parseInt here
     * - parseInt('8azjzekhezka') return 8
     * - parseInt('hkjh8azehrza) return NaN
     * - Number('8azjzekhezka') return NaN
     * - Number('hkjh8azehrza) return NaN
    */
    let extractedNumber = Number(id.replace(stringToRemove, ''));

    if (Number.isNaN(extractedNumber)) {
      throw new Error(EXTRACT_NUMBER_ERROR + id);
    }

    return extractedNumber;
  }

  /**
   * Get all node that have an id that begin with a string.
   *
   * @param {string} string
   * @returns {NodeListOf<Element>}
   * @memberof DOMUtils
   */
  public getAllNodesWithIdThatBeginsWith(string: string): NodeListOf<Element> {
    return document.querySelectorAll('[id^=' + string + ']');
  }
}
