/**
 * Types check utilities.
 *
 * @export
 * @class DataTypesChecker
 */
export default class DataTypesChecker {
  /**
   * Check if the value passed is a string.
   *
   * @static
   * @param {*} value
   * @returns {boolean}
   * @memberof DataTypesChecker
   */
  public static isString(value: any): boolean {
    return typeof value === 'string' || value instanceof String;
  }

  /**
   * Check if the value passed is a number.
   *
   * @static
   * @param {*} value
   * @returns {boolean}
   * @memberof DataTypesChecker
   */
  public static isNumber(value: any): boolean {
    return typeof value === 'number' && Number.isFinite(value);
  }

  /**
   * Check if the value passed is an array.
   *
   * @static
   * @param {*} value
   * @returns {boolean}
   * @memberof DataTypesChecker
   */
  public static isArray(value: any): boolean {
    return value && typeof value === 'object' && value.constructor === Array;
  }

  /**
   * Check if the value passed is a function.
   *
   * @static
   * @param {*} value
   * @returns {boolean}
   * @memberof DataTypesChecker
   */
  public static isFunction(value: any): boolean {
    return typeof value === 'function';
  }

  /**
   * Check if the value passed is an object.
   *
   * @static
   * @param {*} value
   * @returns {boolean}
   * @memberof DataTypesChecker
   */
  public static isObject(value: any): boolean {
    return value && typeof value === 'object' && value.constructor === Object;
  }

  /**
   * Check if the value passed is null.
   *
   * @static
   * @param {*} value
   * @returns {boolean}
   * @memberof DataTypesChecker
   */
  public static isNull(value: any): boolean {
    return value === null;
  }

  /**
   * Check if the value passed is undefined.
   *
   * @static
   * @param {*} value
   * @returns {boolean}
   * @memberof DataTypesChecker
   */
  public static isUndefined(value: any): boolean {
    return typeof value === 'undefined';
  }

  /**
   * Check if the value passed is a boolean.
   *
   * @static
   * @param {*} value
   * @returns {boolean}
   * @memberof DataTypesChecker
   */
  public static isBoolean(value: any): boolean {
    return typeof value === 'boolean';
  }

  /**
   * Check if the value passed is a RegExp.
   *
   * @static
   * @param {*} value
   * @returns {boolean}
   * @memberof DataTypesChecker
   */
  public static isRegExp(value: any): boolean {
    return value && typeof value === 'object' && value.constructor === RegExp;
  }

  /**
   * Check if the value passed is an Error.
   *
   * @static
   * @param {*} value
   * @returns {boolean}
   * @memberof DataTypesChecker
   */
  public static isError(value: any): boolean {
    return value instanceof Error && typeof value.message !== 'undefined';
  }

  /**
   * Check if the value passed is a Date.
   *
   * @static
   * @param {*} value
   * @returns {boolean}
   * @memberof DataTypesChecker
   */
  public static isDate(value: any): boolean {
    return value instanceof Date;
  }

  /**
   * Check if the value passed is a Symbol.
   *
   * @static
   * @param {*} value
   * @returns {boolean}
   * @memberof DataTypesChecker
   */
  public static isSymbol(value: any): boolean {
    return typeof value === 'symbol';
  }
}
