import DOMUtils, { EXTRACT_NUMBER_ERROR } from '../../../src/utils/DOM-utils';

function fakeTheDOM(count: number, idPrefix: string): NodeListOf<Element> {
  let fakeDOM = '';

  for (let index = 0; index < count; index++) {
    fakeDOM += '<div id="' + idPrefix + index + '"></div>';
  }

  document.body.insertAdjacentHTML('afterbegin', fakeDOM);

  return document.querySelectorAll('[id^=' + idPrefix + ']');
}

describe('Datatable jquery component', () => {
  let domUtils = new DOMUtils();

  it(`Extract number from id`, () => {
    let desiredNumber = 1;
    let idPrefix = 'hello-';
    let elementId: string = idPrefix + 1;

    let extractedNumber = domUtils.extractNumberFromId(idPrefix, elementId);

    expect(extractedNumber).toEqual(desiredNumber);
  });

  it(`Extract number from id, should return an error whith a bad id`, () => {
    let idPrefix = 'hello-';
    let elementId: string = idPrefix + '1a';

    expect(() => {
      domUtils.extractNumberFromId(idPrefix, elementId);
    }).toThrowError(EXTRACT_NUMBER_ERROR + elementId);
  });

  it(`Get all nodes with id that begins with a string`, () => {
    let idPrefix = 'hello-';
    let trapPrefix = 'HellO-';
    let instanciatedDom = fakeTheDOM(534, idPrefix);
    fakeTheDOM(45, trapPrefix);

    expect(domUtils.getAllNodesWithIdThatBeginsWith('hello-')).toEqual(
      instanciatedDom
    );
  });

  it(`Extract numbers from ids of nodes`, () => {
    let idPrefix = 'hello-';
    let DOMCount = 534;
    let instanciatedDom = fakeTheDOM(DOMCount, idPrefix);

    let desiredIds: number[] = [];

    for (let index = 0; index < DOMCount; index++) {
      desiredIds.push(index);
    }

    expect(
      domUtils.extractNumbersFromIdsOfNodes(idPrefix, instanciatedDom)
    ).toEqual(desiredIds);
  });

  it(`Extract numbers from ids of nodes should fail with bad prefix`, () => {
    let idPrefix = 'hello-';
    let badPrefix = 'HellO-';
    let instanciatedDom = fakeTheDOM(534, idPrefix);

    expect(() => {
      domUtils.extractNumbersFromIdsOfNodes(badPrefix, instanciatedDom);
    }).toThrowError();
  });

  afterEach(() => {
    document.body.innerHTML = '';
  });
});
