import DataTypesChecker from '../../../src/utils/data-types-checker';

describe('Data types checker', () => {
  let string = 'Hello world!';
  let stringObject = String('Hello world!');
  let integer = 1;
  let float = 1.2;
  let array = [];
  let functionValue = () => null;
  let object = {};
  let nullValue = null;
  let undefinedValue = undefined;
  let boolean = true;
  let regExp = RegExp('.');
  let error = Error('Hello world!');
  let date = new Date(Date.now());
  let symbol = Symbol();

  it(`isString should return true with string and String object`, () => {
    expect(DataTypesChecker.isString(string)).toBeTruthy();
    expect(DataTypesChecker.isString(stringObject)).toBeTruthy();
  });

  it(`isString should return false with other types than string`, () => {
    expect(DataTypesChecker.isString(integer)).toBeFalsy();
    expect(DataTypesChecker.isString(float)).toBeFalsy();
    expect(DataTypesChecker.isString(array)).toBeFalsy();
    expect(DataTypesChecker.isString(functionValue)).toBeFalsy();
    expect(DataTypesChecker.isString(object)).toBeFalsy();
    expect(DataTypesChecker.isString(nullValue)).toBeFalsy();
    expect(DataTypesChecker.isString(undefinedValue)).toBeFalsy();
    expect(DataTypesChecker.isString(boolean)).toBeFalsy();
    expect(DataTypesChecker.isString(regExp)).toBeFalsy();
    expect(DataTypesChecker.isString(date)).toBeFalsy();
    expect(DataTypesChecker.isString(error)).toBeFalsy();
    expect(DataTypesChecker.isString(symbol)).toBeFalsy();
  });

  it(`isNumber should return true with numbers`, () => {
    expect(DataTypesChecker.isNumber(integer)).toBeTruthy();
    expect(DataTypesChecker.isNumber(float)).toBeTruthy();
  });

  it(`isNumber should return false with other types than numbers`, () => {
    expect(DataTypesChecker.isNumber(string)).toBeFalsy();
    expect(DataTypesChecker.isNumber(stringObject)).toBeFalsy();
    expect(DataTypesChecker.isNumber(array)).toBeFalsy();
    expect(DataTypesChecker.isNumber(functionValue)).toBeFalsy();
    expect(DataTypesChecker.isNumber(object)).toBeFalsy();
    expect(DataTypesChecker.isNumber(nullValue)).toBeFalsy();
    expect(DataTypesChecker.isNumber(undefinedValue)).toBeFalsy();
    expect(DataTypesChecker.isNumber(boolean)).toBeFalsy();
    expect(DataTypesChecker.isNumber(regExp)).toBeFalsy();
    expect(DataTypesChecker.isNumber(error)).toBeFalsy();
    expect(DataTypesChecker.isNumber(date)).toBeFalsy();
    expect(DataTypesChecker.isNumber(symbol)).toBeFalsy();
  });

  it(`isArray should return true with arrays`, () => {
    expect(DataTypesChecker.isArray(array)).toBeTruthy();
  });

  it(`isArray should return false with other types than arrays`, () => {
    expect(DataTypesChecker.isArray(integer)).toBeFalsy();
    expect(DataTypesChecker.isArray(float)).toBeFalsy();
    expect(DataTypesChecker.isArray(string)).toBeFalsy();
    expect(DataTypesChecker.isArray(stringObject)).toBeFalsy();
    expect(DataTypesChecker.isArray(functionValue)).toBeFalsy();
    expect(DataTypesChecker.isArray(object)).toBeFalsy();
    expect(DataTypesChecker.isArray(nullValue)).toBeFalsy();
    expect(DataTypesChecker.isArray(undefinedValue)).toBeFalsy();
    expect(DataTypesChecker.isArray(boolean)).toBeFalsy();
    expect(DataTypesChecker.isArray(regExp)).toBeFalsy();
    expect(DataTypesChecker.isArray(error)).toBeFalsy();
    expect(DataTypesChecker.isArray(date)).toBeFalsy();
    expect(DataTypesChecker.isArray(symbol)).toBeFalsy();
  });

  it(`isFunction should return true with functions`, () => {
    expect(DataTypesChecker.isFunction(functionValue)).toBeTruthy();
  });

  it(`isFunction should return false with other types than functions`, () => {
    expect(DataTypesChecker.isFunction(integer)).toBeFalsy();
    expect(DataTypesChecker.isFunction(float)).toBeFalsy();
    expect(DataTypesChecker.isFunction(string)).toBeFalsy();
    expect(DataTypesChecker.isFunction(stringObject)).toBeFalsy();
    expect(DataTypesChecker.isFunction(array)).toBeFalsy();
    expect(DataTypesChecker.isFunction(object)).toBeFalsy();
    expect(DataTypesChecker.isFunction(nullValue)).toBeFalsy();
    expect(DataTypesChecker.isFunction(undefinedValue)).toBeFalsy();
    expect(DataTypesChecker.isFunction(boolean)).toBeFalsy();
    expect(DataTypesChecker.isFunction(regExp)).toBeFalsy();
    expect(DataTypesChecker.isFunction(error)).toBeFalsy();
    expect(DataTypesChecker.isFunction(date)).toBeFalsy();
    expect(DataTypesChecker.isFunction(symbol)).toBeFalsy();
  });

  it(`isObject should return true with objects`, () => {
    expect(DataTypesChecker.isObject(object)).toBeTruthy();
  });

  it(`isObject should return false with other types than objects`, () => {
    expect(DataTypesChecker.isObject(integer)).toBeFalsy();
    expect(DataTypesChecker.isObject(float)).toBeFalsy();
    expect(DataTypesChecker.isObject(string)).toBeFalsy();
    expect(DataTypesChecker.isObject(stringObject)).toBeFalsy();
    expect(DataTypesChecker.isObject(array)).toBeFalsy();
    expect(DataTypesChecker.isObject(functionValue)).toBeFalsy();
    expect(DataTypesChecker.isObject(nullValue)).toBeFalsy();
    expect(DataTypesChecker.isObject(undefinedValue)).toBeFalsy();
    expect(DataTypesChecker.isObject(boolean)).toBeFalsy();
    expect(DataTypesChecker.isObject(regExp)).toBeFalsy();
    expect(DataTypesChecker.isObject(error)).toBeFalsy();
    expect(DataTypesChecker.isObject(date)).toBeFalsy();
    expect(DataTypesChecker.isObject(symbol)).toBeFalsy();
  });

  it(`isNull should return true with null values`, () => {
    expect(DataTypesChecker.isNull(nullValue)).toBeTruthy();
  });

  it(`isNull should return false with other types than null values`, () => {
    expect(DataTypesChecker.isNull(integer)).toBeFalsy();
    expect(DataTypesChecker.isNull(float)).toBeFalsy();
    expect(DataTypesChecker.isNull(string)).toBeFalsy();
    expect(DataTypesChecker.isNull(stringObject)).toBeFalsy();
    expect(DataTypesChecker.isNull(array)).toBeFalsy();
    expect(DataTypesChecker.isNull(functionValue)).toBeFalsy();
    expect(DataTypesChecker.isNull(object)).toBeFalsy();
    expect(DataTypesChecker.isNull(undefinedValue)).toBeFalsy();
    expect(DataTypesChecker.isNull(boolean)).toBeFalsy();
    expect(DataTypesChecker.isNull(regExp)).toBeFalsy();
    expect(DataTypesChecker.isNull(error)).toBeFalsy();
    expect(DataTypesChecker.isNull(date)).toBeFalsy();
    expect(DataTypesChecker.isNull(symbol)).toBeFalsy();
  });

  it(`isUndefined should return true with undefined values`, () => {
    expect(DataTypesChecker.isUndefined(undefinedValue)).toBeTruthy();
  });

  it(`isUndefined should return false with other types than undefined values`, () => {
    expect(DataTypesChecker.isUndefined(integer)).toBeFalsy();
    expect(DataTypesChecker.isUndefined(float)).toBeFalsy();
    expect(DataTypesChecker.isUndefined(string)).toBeFalsy();
    expect(DataTypesChecker.isUndefined(stringObject)).toBeFalsy();
    expect(DataTypesChecker.isUndefined(array)).toBeFalsy();
    expect(DataTypesChecker.isUndefined(functionValue)).toBeFalsy();
    expect(DataTypesChecker.isUndefined(object)).toBeFalsy();
    expect(DataTypesChecker.isUndefined(nullValue)).toBeFalsy();
    expect(DataTypesChecker.isUndefined(boolean)).toBeFalsy();
    expect(DataTypesChecker.isUndefined(regExp)).toBeFalsy();
    expect(DataTypesChecker.isUndefined(error)).toBeFalsy();
    expect(DataTypesChecker.isUndefined(date)).toBeFalsy();
    expect(DataTypesChecker.isUndefined(symbol)).toBeFalsy();
  });

  it(`isBoolean should return true with booleans`, () => {
    expect(DataTypesChecker.isBoolean(boolean)).toBeTruthy();
  });

  it(`isBoolean should return false with other types than booleans`, () => {
    expect(DataTypesChecker.isBoolean(integer)).toBeFalsy();
    expect(DataTypesChecker.isBoolean(float)).toBeFalsy();
    expect(DataTypesChecker.isBoolean(string)).toBeFalsy();
    expect(DataTypesChecker.isBoolean(stringObject)).toBeFalsy();
    expect(DataTypesChecker.isBoolean(array)).toBeFalsy();
    expect(DataTypesChecker.isBoolean(functionValue)).toBeFalsy();
    expect(DataTypesChecker.isBoolean(object)).toBeFalsy();
    expect(DataTypesChecker.isBoolean(nullValue)).toBeFalsy();
    expect(DataTypesChecker.isBoolean(undefinedValue)).toBeFalsy();
    expect(DataTypesChecker.isBoolean(regExp)).toBeFalsy();
    expect(DataTypesChecker.isBoolean(error)).toBeFalsy();
    expect(DataTypesChecker.isBoolean(date)).toBeFalsy();
    expect(DataTypesChecker.isBoolean(symbol)).toBeFalsy();
  });

  it(`isRegExp should return true with booleans`, () => {
    expect(DataTypesChecker.isRegExp(regExp)).toBeTruthy();
  });

  it(`isRegExp should return false with other types than booleans`, () => {
    expect(DataTypesChecker.isRegExp(integer)).toBeFalsy();
    expect(DataTypesChecker.isRegExp(float)).toBeFalsy();
    expect(DataTypesChecker.isRegExp(string)).toBeFalsy();
    expect(DataTypesChecker.isRegExp(stringObject)).toBeFalsy();
    expect(DataTypesChecker.isRegExp(array)).toBeFalsy();
    expect(DataTypesChecker.isRegExp(functionValue)).toBeFalsy();
    expect(DataTypesChecker.isRegExp(object)).toBeFalsy();
    expect(DataTypesChecker.isRegExp(nullValue)).toBeFalsy();
    expect(DataTypesChecker.isRegExp(undefinedValue)).toBeFalsy();
    expect(DataTypesChecker.isRegExp(boolean)).toBeFalsy();
    expect(DataTypesChecker.isRegExp(error)).toBeFalsy();
    expect(DataTypesChecker.isRegExp(date)).toBeFalsy();
    expect(DataTypesChecker.isRegExp(symbol)).toBeFalsy();
  });

  it(`isError should return true with errors`, () => {
    expect(DataTypesChecker.isError(error)).toBeTruthy();
  });

  it(`isError should return false with other types than errors`, () => {
    expect(DataTypesChecker.isError(integer)).toBeFalsy();
    expect(DataTypesChecker.isError(float)).toBeFalsy();
    expect(DataTypesChecker.isError(string)).toBeFalsy();
    expect(DataTypesChecker.isError(stringObject)).toBeFalsy();
    expect(DataTypesChecker.isError(array)).toBeFalsy();
    expect(DataTypesChecker.isError(functionValue)).toBeFalsy();
    expect(DataTypesChecker.isError(object)).toBeFalsy();
    expect(DataTypesChecker.isError(nullValue)).toBeFalsy();
    expect(DataTypesChecker.isError(undefinedValue)).toBeFalsy();
    expect(DataTypesChecker.isError(boolean)).toBeFalsy();
    expect(DataTypesChecker.isError(regExp)).toBeFalsy();
    expect(DataTypesChecker.isError(date)).toBeFalsy();
    expect(DataTypesChecker.isError(symbol)).toBeFalsy();
  });

  it(`isDate should return true with date objects`, () => {
    expect(DataTypesChecker.isDate(date)).toBeTruthy();
  });

  it(`isDate should return false with other types than date objects`, () => {
    expect(DataTypesChecker.isDate(integer)).toBeFalsy();
    expect(DataTypesChecker.isDate(float)).toBeFalsy();
    expect(DataTypesChecker.isDate(string)).toBeFalsy();
    expect(DataTypesChecker.isDate(stringObject)).toBeFalsy();
    expect(DataTypesChecker.isDate(array)).toBeFalsy();
    expect(DataTypesChecker.isDate(functionValue)).toBeFalsy();
    expect(DataTypesChecker.isDate(object)).toBeFalsy();
    expect(DataTypesChecker.isDate(nullValue)).toBeFalsy();
    expect(DataTypesChecker.isDate(undefinedValue)).toBeFalsy();
    expect(DataTypesChecker.isDate(boolean)).toBeFalsy();
    expect(DataTypesChecker.isDate(regExp)).toBeFalsy();
    expect(DataTypesChecker.isDate(error)).toBeFalsy();
    expect(DataTypesChecker.isDate(symbol)).toBeFalsy();
  });

  it(`isSymbol should return true with symbols`, () => {
    expect(DataTypesChecker.isSymbol(symbol)).toBeTruthy();
  });

  it(`isSymbol should return false with other types than symbols`, () => {
    expect(DataTypesChecker.isSymbol(integer)).toBeFalsy();
    expect(DataTypesChecker.isSymbol(float)).toBeFalsy();
    expect(DataTypesChecker.isSymbol(string)).toBeFalsy();
    expect(DataTypesChecker.isSymbol(stringObject)).toBeFalsy();
    expect(DataTypesChecker.isSymbol(array)).toBeFalsy();
    expect(DataTypesChecker.isSymbol(functionValue)).toBeFalsy();
    expect(DataTypesChecker.isSymbol(object)).toBeFalsy();
    expect(DataTypesChecker.isSymbol(nullValue)).toBeFalsy();
    expect(DataTypesChecker.isSymbol(undefinedValue)).toBeFalsy();
    expect(DataTypesChecker.isSymbol(boolean)).toBeFalsy();
    expect(DataTypesChecker.isSymbol(regExp)).toBeFalsy();
    expect(DataTypesChecker.isSymbol(error)).toBeFalsy();
    expect(DataTypesChecker.isSymbol(date)).toBeFalsy();
  });
});
